@@ -21,6 +21,8 @@ define('INGO_STORAGE_ACTION_VACATION', 8);
 define('INGO_STORAGE_ACTION_WHITELIST', 9);
 define('INGO_STORAGE_ACTION_FORWARD', 10);
 define('INGO_STORAGE_ACTION_MOVEKEEP', 11);
+define('INGO_STORAGE_ACTION_FLAGONLY', 12);
+define('INGO_STORAGE_ACTION_NOTIFY', 13);
 
 /**
  * Ingo_Storage:: 'flags' constants
@@ -40,10 +42,10 @@ define('INGO_STORAGE_TYPE_BODY', 3);
 /**
  * Ingo_Storage:: defines an API to store the various filter rules.
  *
- * $Horde: ingo/lib/Storage.php,v 1.43.8.6 2005/07/07 18:31:30 selsky Exp $
+ * $Horde: ingo/lib/Storage.php,v 1.43.8.15 2006/02/01 18:46:42 jan Exp $
  *
- * See the enclosed file LICENSE for license information (ASL).  If you did
- * not receive this file, see http://www.horde.org/licenses/asl.php.
+ * See the enclosed file LICENSE for license information (ASL).  If you
+ * did not receive this file, see http://www.horde.org/licenses/asl.php.
  *
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
  * @since   Ingo 0.1
@@ -52,7 +54,7 @@ define('INGO_STORAGE_TYPE_BODY', 3);
 class Ingo_Storage {
 
     /**
-     * Driver specific parameters
+     * Driver specific parameters.
      *
      * @var array
      */
@@ -99,10 +101,12 @@ class Ingo_Storage {
         require_once dirname(__FILE__) . '/Storage/' . $driver . '.php';
         $class = 'Ingo_Storage_' . $driver;
         if (class_exists($class)) {
-            return $ret = &new $class($params);
+            $ingo = &new $class($params);
         } else {
-            return false;
+            $ingo = false;
         }
+
+        return $ingo;
     }
 
     /**
@@ -156,7 +160,7 @@ class Ingo_Storage {
      *
      * @return Ingo_Storage_rule|Ingo_Storage_filters  The specified object.
      */
-    function retrieve($field, $cache = true)
+    function &retrieve($field, $cache = true)
     {
         if ($cache) {
             if (!isset($this->_cache[$field])) {
@@ -166,28 +170,29 @@ class Ingo_Storage {
                     $cacheSess = &Horde_SessionObjects::singleton();
                     $this->_cache[$field]['ob'] = $cacheSess->query($_SESSION['ingo']['storage'][$field]);
                 } else {
-                    $this->_cache[$field]['ob'] = $this->_retrieve($field);
+                    $this->_cache[$field]['ob'] = &$this->_retrieve($field);
                 }
                 if (!$this->_shutdownCache) {
                     register_shutdown_function(array(&$this, '_addCacheShutdown'));
                     $this->_shutdownCache = true;
                 }
             }
-            return $this->_cache[$field]['ob'];
+            $ob = &$this->_cache[$field]['ob'];
         } else {
-            return $this->_retrieve($field);
+            $ob = &$this->_retrieve($field);
         }
+
+        return $ob;
     }
 
     /**
      * Retrieves the specified data from the storage backend.
      *
      * @abstract
-     *
      * @access private
      *
-     * @param string $field  The field name of the desired data.
-     *                       See lib/Storage.php for the available fields.
+     * @param integer $field  The field name of the desired data.
+     *                        See lib/Storage.php for the available fields.
      *
      * @return Ingo_Storage_rule|Ingo_Storage_filters  The specified data.
      */
@@ -199,13 +204,42 @@ class Ingo_Storage {
     /**
      * Stores the specified data.
      *
-     * @param object $ob      The object to store.
-     * @param boolean $cache  Cache the object?
+     * @param Ingo_Storage_rule|Ingo_Storage_filters $ob  The object to store.
+     * @param boolean $cache                              Cache the object?
      *
      * @return boolean  True on success.
      */
-    function store($ob, $cache = true)
+    function store(&$ob, $cache = true)
     {
+        $type = $ob->obType();
+        if (in_array($type, array(INGO_STORAGE_ACTION_BLACKLIST,
+                                  INGO_STORAGE_ACTION_VACATION,
+                                  INGO_STORAGE_ACTION_WHITELIST,
+                                  INGO_STORAGE_ACTION_FORWARD))) {
+            $filters = $this->retrieve(INGO_STORAGE_ACTION_FILTERS);
+            if ($filters->findRuleId($type) === null) {
+                switch ($type) {
+                case INGO_STORAGE_ACTION_BLACKLIST:
+                    $name = 'Blacklist';
+                    break;
+                case INGO_STORAGE_ACTION_VACATION:
+                    $name = 'Vacation';
+                    break;
+                case INGO_STORAGE_ACTION_WHITELIST:
+                    $name = 'Whitelist';
+                    break;
+                case INGO_STORAGE_ACTION_FORWARD:
+                    $name = 'Forward';
+                    break;
+                }
+                $filters->addRule(array('action' => $type, 'name' => $name));
+                $result = $this->store($filters, $cache);
+                if (is_a($result, 'PEAR_Error')) {
+                    return $result;
+                }
+            }
+        }
+
         if ($cache) {
             $this->_cache[$ob->obType()] = array('ob' => $ob, 'mod' => true);
             if (!$this->_shutdownCache) {
@@ -213,6 +247,7 @@ class Ingo_Storage {
                 $this->_shutdownCache = true;
             }
         }
+
         return $this->_store($ob);
     }
 
@@ -220,16 +255,13 @@ class Ingo_Storage {
      * Stores the specified data in the storage backend.
      *
      * @abstract
-     *
      * @access private
      *
-     * @param string $field  The field name to store the desired data.
-     *                       See lib/Storage.php for the available fields.
-     * @param mixed $data    The data to store.
+     * @param Ingo_Storage_rule|Ingo_Storage_filters $ob  The object to store.
      *
      * @return boolean  True on success.
      */
-    function _store($field, $data)
+    function _store(&$ob)
     {
         return false;
     }
@@ -262,11 +294,11 @@ class Ingo_Storage {
      * @param integer $action  The INGO_STORAGE_ACTION_* value.
      *
      * @return stdClass  Object with the following values:
-     * <code>
+     * <pre>
      * 'flags' => (boolean) Does this action allow flags to be set?
      * 'label' => (string) The label for this action.
      * 'type'  => (string) Either 'folder', 'text', or empty.
-     * </code>
+     * </pre>
      */
     function getActionInfo($action)
     {
@@ -276,7 +308,7 @@ class Ingo_Storage {
 
         switch ($action) {
         case INGO_STORAGE_ACTION_KEEP:
-            $ob->label = _("Deliver into my INBOX");
+            $ob->label = _("Deliver into my Inbox");
             $ob->type = false;
             $ob->flags = true;
             break;
@@ -297,12 +329,12 @@ class Ingo_Storage {
             break;
 
         case INGO_STORAGE_ACTION_REDIRECTKEEP:
-            $ob->label = _("Deliver into my INBOX and redirect to");
+            $ob->label = _("Deliver into my Inbox and redirect to");
             $ob->flags = true;
             break;
 
         case INGO_STORAGE_ACTION_MOVEKEEP:
-            $ob->label = _("Deliver into my INBOX and copy to");
+            $ob->label = _("Deliver into my Inbox and copy to");
             $ob->type = 'folder';
             $ob->flags = true;
             break;
@@ -310,6 +342,16 @@ class Ingo_Storage {
         case INGO_STORAGE_ACTION_REJECT:
             $ob->label = _("Reject with reason");
             break;
+
+        case INGO_STORAGE_ACTION_FLAGONLY:
+            $ob->label = _("Only flag the message");
+            $ob->type = false;
+            $ob->flags = true;
+            break;
+
+        case INGO_STORAGE_ACTION_NOTIFY:
+            $ob->label = _("Notify email address");
+            break;
         }
 
         return $ob;
@@ -318,13 +360,13 @@ class Ingo_Storage {
     /**
      * Returns information on a given test string.
      *
-     * @param mixed $action  The test string.
+     * @param string $action  The test string.
      *
      * @return stdClass  Object with the following values:
-     * <code>
+     * <pre>
      * 'label' => (string) The label for this action.
      * 'type'  => (string) Either 'int', 'none', or 'text'.
-     * </code>
+     * </pre>
      */
     function getTestInfo($test)
     {
@@ -653,8 +695,8 @@ class Ingo_Storage_vacation extends Ingo_Storage_rule {
 }
 
 /**
- * Ingo_Storage_filters is the object used to hold user-defined filtering
- * rule information.
+ * Ingo_Storage_filters is the object used to hold user-defined filtering rule
+ * information.
  *
  * @author  Michael Slusarz <slusarz@bigworm.colorado.edu>
  * @since   Ingo 1.0
@@ -686,21 +728,43 @@ class Ingo_Storage_filters {
         return $this->_obtype;
     }
 
+    /**
+     * Propagates the filter list with data.
+     *
+     * @param array $data  A list of rule hashes.
+     */
     function setFilterlist($data)
     {
         $this->_filters = $data;
     }
 
+    /**
+     * Returns the filter list.
+     *
+     * @return array  The list of rule hashes.
+     */
     function getFilterlist()
     {
         return $this->_filters;
     }
 
+    /**
+     * Returns a single rule hash.
+     *
+     * @param integer $id  A rule number.
+     *
+     * @return array  The requested rule hash.
+     */
     function getRule($id)
     {
         return $this->_filters[$id];
     }
 
+    /**
+     * Returns a rule hash with default value used when creating new rules.
+     *
+     * @return array  A rule hash.
+     */
     function getDefaultRule()
     {
         return array(
@@ -715,15 +779,47 @@ class Ingo_Storage_filters {
         );
     }
 
-    function findRule($action)
+    /**
+     * Searches for the first rule of a certain action type and returns its
+     * number.
+     *
+     * @param integer $action  The field type of the searched rule
+     *                         (INGO_STORAGE_ACTION_* constants).
+     *
+     * @return integer  The number of the first matching rule or null.
+     */
+    function findRuleId($action)
     {
-        foreach ($this->_filters as $rule) {
+        foreach ($this->_filters as $id => $rule) {
             if ($rule['action'] == $action) {
-                return $rule;
+                return $id;
             }
         }
     }
 
+    /**
+     * Searches for and returns the first rule of a certain action type.
+     *
+     * @param integer $action  The field type of the searched rule
+     *                         (INGO_STORAGE_ACTION_* constants).
+     *
+     * @return array  The first matching rule hash or null.
+     */
+    function findRule($action)
+    {
+        $id = $this->findRuleId($action);
+        if ($id !== null) {
+            return $this->getRule($id);
+        }
+    }
+
+    /**
+     * Adds a rule hash to the filters list.
+     *
+     * @param array $rule       A rule hash.
+     * @param boolean $default  If true merge the rule hash with default rule
+     *                          values.
+     */
     function addRule($rule, $default = true)
     {
         if ($default) {
@@ -733,11 +829,24 @@ class Ingo_Storage_filters {
         }
     }
 
+    /**
+     * Updates an existing rule with a rule hash.
+     *
+     * @param array $rule  A rule hash
+     * @param integer $id  A rule number
+     */
     function updateRule($rule, $id)
     {
         $this->_filters[$id] = $rule;
     }
 
+    /**
+     * Deletes a rule from the filters list.
+     *
+     * @param integer $id  Number of the rule to delete.
+     *
+     * @return boolean  True if the rule has been found and deleted.
+     */
     function deleteRule($id)
     {
         if (isset($this->_filters[$id])) {
@@ -749,6 +858,16 @@ class Ingo_Storage_filters {
         return false;
     }
 
+    /**
+     * Creates a copy of an existing rule.
+     *
+     * The created copy is added to the filters list right after the original
+     * rule.
+     *
+     * @param integer $id  Number of the rule to copy.
+     *
+     * @return boolean  True if the rule has been found and copied.
+     */
     function copyRule($id)
     {
         if (isset($this->_filters[$id])) {
@@ -761,41 +880,63 @@ class Ingo_Storage_filters {
         return false;
     }
 
-    function ruleUp($id)
+    /**
+     * Moves a rule up in the filters list.
+     *
+     * @param integer $id     Number of the rule to move.
+     * @param integer $steps  Number of positions to move the rule up.
+     */
+    function ruleUp($id, $steps = 1)
     {
-        while ($id > 0) {
+        for ($i = 0; $i < $steps && $id > 0;) {
             $temp = $this->_filters[$id - 1];
             $this->_filters[$id - 1] = $this->_filters[$id];
             $this->_filters[$id] = $temp;
             /* Continue to move up until we swap with a viewable category. */
             if (in_array($temp['action'], $_SESSION['ingo']['script_categories'])) {
-                break;
+                $i++;
             }
             $id--;
         }
     }
 
-    function ruleDown($id)
+    /**
+     * Moves a rule down in the filters list.
+     *
+     * @param integer $id     Number of the rule to move.
+     * @param integer $steps  Number of positions to move the rule down.
+     */
+    function ruleDown($id, $steps = 1)
     {
         $rulecount = count($this->_filters) - 1;
-        while ($id < $rulecount) {
+        for ($i = 0; $i < $steps && $id < $rulecount;) {
             $temp = $this->_filters[$id + 1];
             $this->_filters[$id + 1] = $this->_filters[$id];
             $this->_filters[$id] = $temp;
             /* Continue to move down until we swap with a viewable
                category. */
             if (in_array($temp['action'], $_SESSION['ingo']['script_categories'])) {
-                break;
+                $i++;
             }
             $id++;
         }
     }
 
+    /**
+     * Disables a rule.
+     *
+     * @param integer $id  Number of the rule to disable.
+     */
     function ruleDisable($id)
     {
         $this->_filters[$id]['disable'] = true;
     }
 
+    /**
+     * Enables a rule.
+     *
+     * @param integer $id  Number of the rule to enable.
+     */
     function ruleEnable($id)
     {
         $this->_filters[$id]['disable'] = false;
